@@ -333,6 +333,7 @@ class SeeqNames:
         graph_q_l = '/graphql'
         class ErrorMessages:
             attempted_to_set_scope_on_a_globally_scoped_item = 'Attempted to set scope on a globally scoped item'
+            cancelled_due_to_too_many_retries = 'The request was canceled due to being retried too many times'
         
         class AddOnToolLinkType:
             window = 'window'
@@ -731,6 +732,7 @@ class SeeqNames:
         admin = 'Admin'
         workbench = 'Workbench'
         user_language = 'userLanguage'
+        agent_a_p_i_key_permissions_migrated = 'Agent API Key Permissions Migrated'
         agent_provisioning_status = 'Agent Provisioning Status'
         agent_provisioning_source_address = 'Agent Provisioning Source Address'
         agent_provisioning_requested_at = 'Agent Provisioning Requested At'
