@@ -1757,8 +1757,17 @@ def push_access_control(session: Session, item_id: str, acl_df: pd.DataFrame, re
         ace_inputs.append((permissions, ace_to_push['ID']))
 
     if replace:
-        # It's important to remove the entries that need to be removed before adding. Otherwise, if you try to
-        # add an ACE that conflicts with an existing entry, it will be silently ignored.
+        # We need to add the requested permissions first even if in some cases these will be silently ignored because
+        # manage permission can be inherited from a group and by removing the respective permissions we lose the
+        # ability to manage the permissions (e.g. agent_api_key gets the manage permission from Agents group in R63+).
+        # Any silently-ignored additions will be correctly added after existing ACE entries are removed.
+        for permissions, identity_id in ace_inputs:
+            items_api.add_access_control_entry(id=item_id,
+                                               body=AceInputV1(permissions=permissions,
+                                                               identity_id=identity_id))
+
+        # It's important to remove the entries that need to be removed before adding the final permissions. Otherwise,
+        # if you try to add an ACE that conflicts with an existing entry, it will be silently ignored.
         for existing_ace in acl_output.entries:  # type: AceOutputV1
             if existing_ace.role == 'OWNER' or (not disable_permission_inheritance and existing_ace.origin is not None):
                 # You can't remove OWNER or inherited permissions
