@@ -20,6 +20,7 @@ from seeq.spy._status import Status
 from seeq.spy.addons import _permissions
 
 
+@Status.handle_keyboard_interrupt()
 def install(tool: Union[pd.DataFrame, pd.Series, dict, list], *, include_workbook_parameters: bool = True,
             update_tool: bool = False, update_permissions: bool = False, errors: Optional[str] = None,
             quiet: Optional[bool] = None, status: Optional[Status] = None, session: Optional[Session] = None
@@ -166,7 +167,7 @@ def install(tool: Union[pd.DataFrame, pd.Series, dict, list], *, include_workboo
     quiet : bool, default False
         If True, suppresses progress output. Note that when status is
         provided, the quiet setting of the Status object that is passed
-        in takes precedent.
+        in takes precedence.
 
     status : spy.Status, optional
         If specified, the supplied Status object will be updated as the command
