@@ -1083,7 +1083,7 @@ class ItemGroup(list):
                     [_common.safe_json_dumps(p) for p in parameters.values()])
                 raise SPyRuntimeError('Cannot compute statistics across different types of items:\n%s' % debug_string)
 
-            _type = types.pop().replace('Stored', '').replace('Calculated', '')
+            _type = types.pop().replace('Stored', '').replace('Calculated', '').replace('Literal', '')
             fns = {fn.statistic: fn for fn in _common.ROLL_UP_FUNCTIONS if fn.input_type == _type}
 
             if statistic.lower() not in fns:
