@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import pandas as pd
 
-reference_types = ['StoredSignal', 'StoredCondition']
+reference_types = ['LiteralScalar', 'StoredSignal', 'StoredCondition']
 calculated_types = ['CalculatedScalar', 'CalculatedSignal', 'CalculatedCondition']
 metric_types = ['Metric', 'ThresholdMetric']
 data_types = calculated_types + reference_types
