@@ -97,7 +97,7 @@ class Tree:
         operations on this Tree. This option can be changed later using
         `tree.quiet = True` or by specifying the option for individual function calls.
         Note that when status is provided, the quiet setting of the Status object
-        that is passed in takes precedent.
+        that is passed in takes precedence.
 
     errors : {'raise', 'catalog'}, default 'raise'
         If 'raise', any errors encountered will cause an exception. If 'catalog',
@@ -127,6 +127,7 @@ class Tree:
     status: Status
     is_dirty = False
 
+    @Status.handle_keyboard_interrupt()
     def __init__(self, data, *, friendly_name=None, description=None, workbook=_common.DEFAULT_WORKBOOK_PATH,
                  datasource=None, convert_displays_to_sdl=True, quiet=None, errors=None, status=None,
                  session: Optional[Session] = None):
@@ -279,6 +280,7 @@ class Tree:
         self._dataframe = df
         self.status.update(f'{status_message} {self.summarize(ret=True)}', Status.SUCCESS)
 
+    # @Status.handle_keyboard_interrupt()
     def insert(self, children=None, parent=None, *, name=None, formula=None, formula_parameters=None,
                roll_up_statistic=None, roll_up_parameters=None, friendly_name=None, errors=None, quiet=None,
                status=None):
@@ -323,7 +325,12 @@ class Tree:
             used in place of the `children` argument.
 
         formula_parameters : dict, optional
-            The parameters for a formula.
+            The parameters used in the specified `formula`. The following options are allowed:
+
+            1) The name of an item that is an asset sibling of this calculation.
+            2) The relative path to an item in this Tree (E.G. '.. >> Area E >> Temperature').
+            3) The absolute path to an item in this Tree (E.G. 'Example >> Cooling Tower 2 >> Area E >> Temperature').
+            4) The ID of an item (including ones not in the Tree).
 
         roll_up_statistic : str, optional
             The statistic to use when inserting a roll-up calculation. Valid options are
@@ -351,7 +358,7 @@ class Tree:
             If True, suppresses progress output. This input will be used only for the
             duration of this function; it will default to the setting on the Tree if
             not specified. Note that when status is provided, the quiet setting of
-            the Status object that is passed in takes precedent.
+            the Status object that is passed in takes precedence.
 
         status : spy.Status, optional
             If specified, the supplied Status object will be updated as the command
@@ -375,7 +382,7 @@ class Tree:
             (status, 'status', Status)
         ])
 
-        status = self.validate_status(status, quiet, errors)
+        status = Status.validate(status, quiet, errors)
 
         if children is None:
             names = [arg for arg in (name, friendly_name) if arg is not None]
@@ -467,7 +474,7 @@ class Tree:
         additions = list()
         if 'Parent' in children.columns:
             for node, matched_indices in _match.Query(self._dataframe).multimatch(children['Parent']):
-                children_to_add = _path.set_children_path(children.loc[matched_indices].copy(), node.full_path,
+                children_to_add = _path.set_children_path(children.loc[list(matched_indices)].copy(), node.full_path,
                                                           node.depth)
                 additions.append(children_to_add)
         else:
@@ -526,7 +533,7 @@ class Tree:
             If True, suppresses progress output. This input will be used only for the
             duration of this function; it will default to the setting on the Tree if
             not specified. Note that when status is provided, the quiet setting of
-            the Status object that is passed in takes precedent.
+            the Status object that is passed in takes precedence.
 
         status : spy.Status, optional
             If specified, the supplied Status object will be updated as the command
@@ -543,7 +550,7 @@ class Tree:
             (status, 'status', Status)
         ])
 
-        status = self.validate_status(status, quiet, errors)
+        status = Status.validate(status, quiet, errors)
 
         _utils.initialize_status_df(status, 'Removed', 'Removing items from tree', Status.RUNNING)
 
@@ -597,7 +604,7 @@ class Tree:
             If True, suppresses progress output. This input will be used only for the
             duration of this function; it will default to the setting on the Tree if
             not specified. Note that when status is provided, the quiet setting of
-            the Status object that is passed in takes precedent.
+            the Status object that is passed in takes precedence.
 
         status : spy.Status, optional
             If specified, the supplied Status object will be updated as the command
@@ -615,7 +622,7 @@ class Tree:
             (status, 'status', Status)
         ])
 
-        status = self.validate_status(status, quiet, errors)
+        status = Status.validate(status, quiet, errors)
 
         _utils.initialize_status_df(status, 'Moved', 'Moving items in tree.', Status.RUNNING)
 
@@ -970,7 +977,7 @@ class Tree:
             If True, suppresses progress output. This input will be used only for the
             duration of this function; it will default to the setting on the Tree if
             not specified. Note that when status is provided, the quiet setting of
-            the Status object that is passed in takes precedent.
+            the Status object that is passed in takes precedence.
 
         status : spy.Status, optional
             If specified, the supplied Status object will be updated as the command
@@ -995,7 +1002,7 @@ class Tree:
             (status, 'status', Status)
         ])
 
-        status = self.validate_status(status, quiet, errors)
+        status = Status.validate(status, quiet, errors)
 
         if condition is None and within is None:
             raise SPyValueError("At least one of `condition` or `within` arguments must be provided")
@@ -1039,6 +1046,7 @@ class Tree:
 
         return result
 
+    @Status.handle_keyboard_interrupt()
     def push(self, *, metadata_state_file: Optional[str] = None, errors: Optional[str] = None,
              quiet: Optional[bool] = None, status: Optional[Status] = None) -> pd.DataFrame:
         """
@@ -1062,7 +1070,7 @@ class Tree:
             If True, suppresses progress output. This input will be used only for the
             duration of this function; it will default to the setting on the Tree if
             not specified. Note that when status is provided, the quiet setting of
-            the Status object that is passed in takes precedent.
+            the Status object that is passed in takes precedence.
 
         status : spy.Status, optional
             If specified, the supplied Status object will be updated as the command
@@ -1084,7 +1092,7 @@ class Tree:
             (status, 'status', Status)
         ])
 
-        status = self.validate_status(status, quiet, errors)
+        status = Status.validate(status, quiet, errors)
 
         if self._workbook is not None:
             self._push_workbook()
@@ -1142,22 +1150,11 @@ class Tree:
             self._workbook_id = _common.sanitize_guid(self._workbook)
         elif self.session.client:
             search_query, _ = _push.create_analysis_search_query(self._workbook)
-            search_df = spy.workbooks.search(search_query,
-                                             status=status,
-                                             session=self.session)
+            search_df = spy.workbooks.search(search_query, status=status, session=self.session)
             self._workbook_id = search_df.iloc[0]['ID'] if len(search_df) > 0 else _constants.UNKNOWN
         else:
             self._workbook_id = _constants.UNKNOWN
 
-    def validate_status(self, status: Status, quiet: Optional[bool], errors: Optional[str]):
-        if status is None:
-            status = Status()
-
-        status.errors = errors if errors is not None else self.status.errors
-        status.quiet = quiet if quiet is not None else self.status.quiet
-
-        return status
-
 
 class DownSelectedTree(Tree):
     # noinspection PyMissingConstructor
