@@ -6,9 +6,11 @@ import re
 from typing import List, Dict, Union, Optional
 
 import pandas as pd
+
+from seeq import spy
 from seeq.base import util
 from seeq.sdk import *
-from seeq.spy import _common, _metadata, _search
+from seeq.spy import _common, _metadata
 from seeq.spy._errors import *
 from seeq.spy._redaction import safely
 from seeq.spy._session import Session
@@ -16,8 +18,8 @@ from seeq.spy.workbooks._item import Item, ItemMap
 
 
 class StoredOrCalculatedItem(Item):
-    def __init__(self, definition=None):
-        super().__init__(definition)
+    def __init__(self, definition=None, *, provenance=None):
+        super().__init__(definition, provenance=provenance)
 
         self.scoped_to = None
 
@@ -31,15 +33,8 @@ class StoredOrCalculatedItem(Item):
             return
 
         ancestors = [a.name for a in asset_tree_output.item.ancestors]
-        if len(ancestors) >= 2:
-            self.definition['Path'] = _common.path_list_to_string(ancestors[0:-1])
-            self.definition['Asset'] = ancestors[-1]
-        elif len(ancestors) == 1:
-            self.definition['Path'] = None
-            self.definition['Asset'] = ancestors[0]
-        else:
-            self.definition['Path'] = None
-            self.definition['Asset'] = None
+
+        _common.add_ancestors_to_definition(self.type, self.name, ancestors, self.definition, old_asset_format=False)
 
     def _pull(self, session: Session, item_id, status):
         super()._pull(session, item_id, status)
@@ -302,6 +297,7 @@ class StoredOrCalculatedItem(Item):
 
         if new_definition['Type'] not in ['User', 'UserGroup']:
             query_df = pd.DataFrame([new_definition])
+            from seeq.spy import _search
             if 'Path' in new_definition:
                 # If we're matching based on path, unfortunately we will not be able to match against archived items
                 # due to a limitation of the API. See CRAB-18439.
@@ -382,7 +378,19 @@ class StoredItem(StoredOrCalculatedItem):
 
 
 class Asset(StoredItem):
-    pass
+    def __init__(self, definition=None, *, provenance=None):
+        super().__init__(definition, provenance=provenance)
+
+        if self.provenance == Item.LOAD:
+            if self.definition.get('Old Asset Format', True):
+                if self.type == 'Asset':
+                    if self.definition.get('Path') in [None, '']:
+                        self.definition['Path'] = self.definition['Asset']
+                    else:
+                        self.definition['Path'] += ' >> ' + self.definition['Asset']
+                        self.definition['Asset'] = self.definition['Name']
+
+        self.definition['Old Asset Format'] = False
 
 
 class Datasource(StoredItem):
@@ -397,10 +405,6 @@ class Datasource(StoredItem):
         })
 
 
-class TableDatasource(Datasource):
-    pass
-
-
 class CalculatedItem(StoredOrCalculatedItem):
     def push(self, session: Session, datasource_maps, datasource_output, *, pushed_workbook_id=None,
              item_map: ItemMap = None, label=None, override_max_interp=False, scope_globals_to_workbook=False):
@@ -478,10 +482,13 @@ class CalculatedItem(StoredOrCalculatedItem):
         return item_input
 
     def _set_ui_config(self, session: Session, _id):
-        items_api = ItemsApi(session.client)
         if 'UIConfig' in self.definition:
-            items_api.set_property(id=_id, property_name='UIConfig',
-                                   body=PropertyInputV1(value=json.dumps(self.definition['UIConfig'])))
+            try:
+                items_api = ItemsApi(session.client)
+                items_api.set_property(id=_id, property_name='UIConfig',
+                                       body=PropertyInputV1(value=json.dumps(self.definition['UIConfig'])))
+            except ValueError:  # ValueError includes JSONDecodeError, but ApiException will not be caught
+                pass
 
 
 class StoredSignal(StoredItem):
@@ -588,6 +595,10 @@ class CalculatedCondition(CalculatedItem):
         return item_update_output.item
 
 
+class LiteralScalar(StoredItem):
+    pass
+
+
 class CalculatedScalar(CalculatedItem):
     def _pull(self, session: Session, item_id, status):
         super()._pull(session, item_id, status)
@@ -619,10 +630,19 @@ class CalculatedScalar(CalculatedItem):
             datasource_class = datasource_output.datasource_class
             datasource_id = datasource_output.datasource_id
             scalar_input.data_id = self._construct_data_id(label)
-            item_batch_output = scalars_api.put_scalars(
-                body=PutScalarsInputV1(datasource_class=datasource_class,
-                                       datasource_id=datasource_id,
-                                       scalars=[scalar_input]))  # type: ItemBatchOutputV1
+
+            if (spy.utils.is_sdk_module_version_at_least(61, 1, 4) and not spy.utils.is_sdk_module_version_at_least(
+                    62)) or spy.utils.is_sdk_module_version_at_least(62, 0, 1):
+                body = PutScalarsInputV1(datasource_class=datasource_class,
+                                         datasource_id=datasource_id,
+                                         scalars=[scalar_input],
+                                         force_calculated_scalars=True)
+            else:
+                body = PutScalarsInputV1(datasource_class=datasource_class,
+                                         datasource_id=datasource_id,
+                                         scalars=[scalar_input])
+
+            item_batch_output = scalars_api.put_scalars(body=body)  # type: ItemBatchOutputV1
 
             item_update_output = item_batch_output.item_updates[0]  # type: ItemUpdateOutputV1
             if item_update_output.error_message is not None:
