@@ -4,11 +4,12 @@ import copy
 import glob
 import json
 import os
+import shutil
 from typing import Dict, List, Optional, Union
 
 import numpy as np
 
-from seeq.base import system
+from seeq.base import util
 from seeq.base.seeq_names import SeeqNames
 from seeq.sdk import *
 from seeq.spy import _common
@@ -51,11 +52,11 @@ class Workbook(ItemWithOwnerAndAcl):
 
         return object.__new__(cls)
 
-    def __init__(self, definition=None):
+    def __init__(self, definition=None, *, provenance=None):
         if isinstance(definition, str):
             definition = {'Name': definition}
 
-        super().__init__(definition)
+        super().__init__(definition, provenance=provenance)
 
         self.status = None
         self._item_inventory = dict()
@@ -122,7 +123,7 @@ class Workbook(ItemWithOwnerAndAcl):
 
     def _get_errors_str(self, errors):
         return 'Errors encountered:\n' + \
-               '\n----------------------------------------\n'.join(errors)
+            '\n----------------------------------------\n'.join(errors)
 
     def update_status(self, result, count_increment):
         if self.status is None:
@@ -204,7 +205,7 @@ class Workbook(ItemWithOwnerAndAcl):
             self.status = None
 
     @staticmethod
-    def _instantiate(definition=None):
+    def _instantiate(definition=None, *, provenance=None):
         if definition['Type'] == 'Workbook':
             if 'Workbook Type' not in definition:
                 if 'Data' not in definition or _common.get_workbook_type(definition['Data']) == 'Analysis':
@@ -221,9 +222,9 @@ class Workbook(ItemWithOwnerAndAcl):
             raise SPyValueError(f"Unrecognized workbook type: {definition['Type']}")
 
         if definition['Workbook Type'] == 'Analysis':
-            return Analysis(definition)
+            return Analysis(definition, provenance=provenance)
         elif definition['Workbook Type'] == 'Topic':
-            return Topic(definition)
+            return Topic(definition, provenance=provenance)
 
     @staticmethod
     def pull(item_id, *, status=None, extra_workstep_tuples=None, include_inventory=True, include_annotations=True,
@@ -237,7 +238,7 @@ class Workbook(ItemWithOwnerAndAcl):
         if item_output is None:
             return
         definition = Item._dict_from_item_output(item_output)
-        workbook = Workbook._instantiate(definition)
+        workbook = Workbook._instantiate(definition, provenance=Item.PULL)
         workbook._pull(session, workbook_id=workbook.id, extra_workstep_tuples=extra_workstep_tuples,
                        include_inventory=include_inventory, include_annotations=include_annotations,
                        include_images=include_images, specific_worksheet_ids=specific_worksheet_ids, status=status)
@@ -264,8 +265,6 @@ class Workbook(ItemWithOwnerAndAcl):
             self._definition['Path'] = _common.path_list_to_string([a.name for a in workbook_output.ancestors])
             self._definition['Workbook Type'] = _common.get_workbook_type(workbook_output)
 
-            self._provenance = Item.PULL
-
             self._pull_owner_and_acl(session, workbook_output.owner, self.status)
             self._pull_ancestors(session, workbook_output.ancestors)
 
@@ -417,46 +416,8 @@ class Workbook(ItemWithOwnerAndAcl):
                     items_api.set_properties(id=workbook_output.id,
                                              body=[ScalarPropertyV1(name='Archived', value=False)])
 
-                for is_archived in [False, True]:
-                    offset = 0
-                    limit = 1000
-                    while True:
-                        worksheet_output_list = safely(
-                            lambda: workbooks_api.get_worksheets(workbook_id=workbook_output.id,
-                                                                 is_archived=is_archived,
-                                                                 offset=offset,
-                                                                 limit=limit),
-                            action_description=f'get worksheets for workbook {workbook_output.id}',
-                            status=self.status)  # type: WorksheetOutputListV1
-                        if worksheet_output_list is None:
-                            break
-
-                        for worksheet_output in worksheet_output_list.worksheets:  # type: WorksheetOutputV1
-                            @request_safely(
-                                action_description=f'get Data ID for worksheet '
-                                                   f'{workbook_output.id}/{worksheet_output.id}',
-                                status=self.status)
-                            def _add_worksheet_data_id_to_identifiers():
-                                item_output = items_api.get_item_and_all_properties(
-                                    id=worksheet_output.id)  # type: ItemOutputV1
-                                data_id = [p.value for p in item_output.properties if p.name == 'Data ID']
-                                spy_id = [p.value for p in item_output.properties if p.name == 'SPy ID']
-                                # This is for backward compatibility with worksheets that had been pushed by SPy with a
-                                # Data ID. (We switched to "SPy ID" because there's no use in using the Datasource Class
-                                # / Datasource ID / Data ID triplet when you can't actually search on it.)
-                                if len(data_id) != 0:
-                                    existing_worksheet_identifiers[data_id[0]] = worksheet_output.id
-                                elif len(spy_id) != 0:
-                                    existing_worksheet_identifiers[spy_id[0]] = worksheet_output.id
-
-                            existing_worksheet_identifiers[worksheet_output.id] = worksheet_output.id
-                            existing_worksheet_identifiers[worksheet_output.name] = worksheet_output.id
-                            _add_worksheet_data_id_to_identifiers()
-
-                        if len(worksheet_output_list.worksheets) < limit:
-                            break
-
-                        offset = offset + limit
+                if specific_worksheet_ids is None or len(specific_worksheet_ids) > 0:
+                    existing_worksheet_identifiers = self._get_existing_worksheet_identifiers(session, workbook_output)
 
                 owner_id = self.decide_owner(session, self.datasource_maps, item_map, owner=owner,
                                              current_owner_id=workbook_output.owner.id)
@@ -545,7 +506,7 @@ class Workbook(ItemWithOwnerAndAcl):
                     if specific_worksheet_ids is not None and worksheet.id not in specific_worksheet_ids:
                         continue
 
-                    dependencies_not_found.update(worksheet.find_unresolved_worksteps(item_map))
+                    dependencies_not_found.update(worksheet.find_unresolved_worksteps())
 
             if first_worksheet_id is not None:
                 link_url = '%s/%sworkbook/%s/worksheet/%s' % (
@@ -565,6 +526,53 @@ class Workbook(ItemWithOwnerAndAcl):
         finally:
             self.status = None
 
+    def _get_existing_worksheet_identifiers(self, session: Session, workbook_output: WorkbookOutputV1) -> dict:
+        workbooks_api = WorkbooksApi(session.client)
+        items_api = ItemsApi(session.client)
+        existing_worksheet_identifiers = dict()
+        for is_archived in [False, True]:
+            offset = 0
+            limit = 1000
+            while True:
+                worksheet_output_list = safely(
+                    lambda: workbooks_api.get_worksheets(workbook_id=workbook_output.id,
+                                                         is_archived=is_archived,
+                                                         offset=offset,
+                                                         limit=limit),
+                    action_description=f'get worksheets for workbook {workbook_output.id}',
+                    status=self.status)  # type: WorksheetOutputListV1
+                if worksheet_output_list is None:
+                    break
+
+                for worksheet_output in worksheet_output_list.worksheets:  # type: WorksheetOutputV1
+                    @request_safely(
+                        action_description=f'get Data ID for worksheet '
+                                           f'{workbook_output.id}/{worksheet_output.id}',
+                        status=self.status)
+                    def _add_worksheet_data_id_to_identifiers():
+                        item_output = items_api.get_item_and_all_properties(
+                            id=worksheet_output.id)  # type: ItemOutputV1
+                        data_id = [p.value for p in item_output.properties if p.name == 'Data ID']
+                        spy_id = [p.value for p in item_output.properties if p.name == 'SPy ID']
+                        # This is for backward compatibility with worksheets that had been pushed by SPy with a
+                        # Data ID. (We switched to "SPy ID" because there's no use in using the Datasource Class
+                        # / Datasource ID / Data ID triplet when you can't actually search on it.)
+                        if len(data_id) != 0:
+                            existing_worksheet_identifiers[data_id[0]] = worksheet_output.id
+                        elif len(spy_id) != 0:
+                            existing_worksheet_identifiers[spy_id[0]] = worksheet_output.id
+
+                    existing_worksheet_identifiers[worksheet_output.id] = worksheet_output.id
+                    existing_worksheet_identifiers[worksheet_output.name] = worksheet_output.id
+                    _add_worksheet_data_id_to_identifiers()
+
+                if len(worksheet_output_list.worksheets) < limit:
+                    break
+
+                offset = offset + limit
+
+        return existing_worksheet_identifiers
+
     def _push_inventory(self, session: Session, item_map: ItemMap, label, datasource_output, override_max_interp,
                         scope_globals_to_workbook, workbook_output):
         remaining_inventory = dict(self.item_inventory)
@@ -691,7 +699,7 @@ class Workbook(ItemWithOwnerAndAcl):
     @staticmethod
     def _get_datasource_map_json_file(workbook_folder, datasource_map):
         return os.path.join(
-            workbook_folder, system.cleanse_filename(
+            workbook_folder, util.cleanse_filename(
                 'Datasource_Map_%s_%s_%s.json' % (datasource_map['Datasource Class'],
                                                   datasource_map['Datasource ID'],
                                                   datasource_map['Datasource Name'])))
@@ -703,7 +711,11 @@ class Workbook(ItemWithOwnerAndAcl):
 
         if os.path.exists(workbook_folder):
             if overwrite:
-                system.removetree(workbook_folder, keep_top_folder=True)
+                for root, dirs, files in os.walk(workbook_folder):
+                    for _file in files:
+                        os.unlink(os.path.join(root, _file))
+                    for _dir in dirs:
+                        shutil.rmtree(os.path.join(root, _dir))
             else:
                 raise SPyRuntimeError('"%s" folder exists. Use shutil.rmtree() to remove it, but be careful not to '
                                       'accidentally delete your work!' % workbook_folder)
@@ -753,14 +765,12 @@ class Workbook(ItemWithOwnerAndAcl):
         with open(workbook_json_file, 'r', encoding='utf-8') as f:
             definition = json.load(f)
 
-        workbook = Workbook._instantiate(definition)
+        workbook = Workbook._instantiate(definition, provenance=Item.LOAD)
         workbook._load(workbook_folder)
 
         return workbook
 
     def _load(self, workbook_folder):
-        self._provenance = Item.LOAD
-
         self.worksheets = WorksheetList(self)
         for worksheet_id in self.definition['Worksheets']:
             Worksheet.load_from_workbook_folder(self, workbook_folder, worksheet_id)
@@ -913,10 +923,10 @@ class Workbook(ItemWithOwnerAndAcl):
             'CalculatedSignal',
             'StoredCondition',
             'CalculatedCondition',
+            'LiteralScalar',
             'CalculatedScalar',
             'Chart',
-            'ThresholdMetric',
-            'TableDatasource'
+            'ThresholdMetric'
         ]
 
         self.update_status('Scraping item inventory', 0)
@@ -949,7 +959,7 @@ class Workbook(ItemWithOwnerAndAcl):
             self.update_status('Scraping item dependency', 0)
 
             try:
-                dep_item = Item.pull(dependency.id, allowed_types=allowed_types, session=session)
+                dep_item = Item.pull(dependency.id, allowed_types=allowed_types, session=session, status=self.status)
             except ApiException as e:
                 if e.status == 404:
                     continue
@@ -1100,10 +1110,10 @@ class Topic(Workbook):
             timer = _common.timer_start()
             worksheet.pull_rendered_content(session, status=status.create_inner(f'Pull Embedded Content {worksheet}'))
             status.df.at[worksheet.id, 'Name'] = worksheet.name
-            if worksheet.document.rendered_content_images is None:
+            if worksheet.report.rendered_content_images is None:
                 status.df.at[worksheet.id, 'Count'] = np.nan
             else:
-                status.df.at[worksheet.id, 'Count'] = len(worksheet.document.rendered_content_images)
+                status.df.at[worksheet.id, 'Count'] = len(worksheet.report.rendered_content_images)
             status.df.at[worksheet.id, 'Time'] = _common.timer_elapsed(timer)
 
 
